/*************************************************************
Jarvis Schultz and Jake Ware
3-9-2010

This source file contains the primary code that was written for 
controlling a differential drive mobile robot that was created as a 
replacement for the e-puck for swarm robotics research.  The user 
sends serial commands wirelessly through the use of an XBee 
(1 stop bit, no parity bits, no flow control, 115200 baud), and 
this library can interpret the commands.  It is designed to work 
on a PIC32.

Each string that the user sends to the robot contains fifteen 
eight bit characters.  The most significant character is basically
a header character that lets the robot know what kind of instruction 
it is receiving.  

While the robot is executing the received commands a timer 2 interrupt
service routine is called at some set frequency that updates the robot's
current pose using dead reckoning.  The dead reckoning provides the location
(in x and y coordinates) of the center of the robot, and the angle between the
front of the robot and the positive x-axis (ccw rotation is considered to be 
positive, and the angle is in the range of 0-2pi radians).
The code is designed to be almost entirely inerrupt based.  
This leaves the CPU mostly free to be doing other types of calculations.

There are two overarching "modes" of operation.  In the first mode,
the user simply specifies the direction of rotation and speed that is
desired for each wheel.  This mode is useful if all of the commands 
that are desired for the wheels are being executed on user's end. 
The second mode allows the user to simply specify a desired position and 
orientation that the robot should acheive, and the robot will use 
its dead reckoning in order to drive to that configuration.  During 
either of these modes of operation, the user also has the ability to 
send additional information to the robot.  Corrected position and 
orientation can be sent; this is useful if, for example, a camera is
being used as a feedback device.  Also, the user can send an instruction
that stops the robot where it is and defines its current location as the
new origin of the global coordinate system.

Below is a summary of each instuction that the user can send along with
information about what the robot will do when it receives the instruction.

1)  Configuration Control:
	This is the mode described above where the user specifies a desired 
	configuration for the robot, and it begins driving there.  The motion
	is broken into three sections, an initial rotation that gets the robot 
	oriented such that it can begin driving towards the final location, a
	straight drive towards the final location, and a final rotation to
	the final desired orientation.  The robot decides which direction to
	rotate at both ends and whether to drive forwards or backwards based 
	on which combination requires the least distance of driving.  If while
	executing one of these point commands, a new instruction is received, 
	the robot immediately disregards the old instruction and starts the new.
	This previous statement is true regardless of what type the new instruction 
	is.  If the robot receives an update as to what its current configuration
	is while executing the configuration command, it immediately recalculates 
	what it must do to finish the command and then continues until it is done.
	The string that should be sent contains the following characters:
	Char 1 -			'p' this tells the robot that this command is a point command
	Char 2 -			'0' or '1' this tells the robot the sign of the x-location desired
							a zero indicates negative, and a one inicates positive
	Char 3,4,5,6 - 		These four characters are the desired x-location in inches.  Char 3 is the
							tens digit, 4 is the ones digit, 5 is the tenths, and 6 is the hundredths
	Char 7 -			'0' or '1' this tells the robot the sign of the y-location desired
							a zero indicates negative, and a one inicates positive
	Char 8,9,10,11-		These four characters are the desired y-location in inches.  Char 3 is the
							tens digit, 4 is the ones digit, 5 is the tenths, and 6 is the hundredths
	Char 12,13,14,15 -	These final four characters are the desired final orientation (between 0 and 
							2pi radians).  Char 12 is the ones digit, and the final three are the 
							tenths, hundredths, and thousandths respectively.
							
2) Wheel Speed Control:
	In this mode, the user simply specifies the direction and speed of each wheel. 
	The robot will continue performing the desired action until told to stop or change.
	The string sent by the user is as follows:
	Char 1 -			'h' This char tells the robot that this is a speed instruction
	Char 2 - 			'0' or '1' direction of the left wheel, a one makes the robot go forward and a zero
							backwards
	Char 3,4,5 - 		The next three chars are the speed of the left wheel in steps per second the maximum
							is probably about 450
	Char 6 - 			'0' or '1' direction of the right wheel, a one makes the robot go forward and a zero
							backwards
	Char 7,8,9 - 		The next three chars are the speed of the right wheel in steps per second the maximum
							is probably about 450							
	The remaining chars can be anything, but they need to be something so that we fill the 15 in-buffer.
	
3) Corrected Configuration:
	This simply changes what the robot thinks its current configuration is.  Useful for setting the robot's initial
	location at something other than (0,0,0), or correcting its dead reckoning with feedback.  The first char is an
	'l', and the rest of the string follows the exact same format as the configuration control mode.
	
4) Reset Coordinate System:
	This stops the robot and resets its odometry (effectively translating the global coordinate system to the 
	robot's center and rotating it to align then positive x-axis with the front of the robot).  This is an 'r'
	followed by 14 random chars.
	
5) Change default speed:
	When running location control, the robot simply uses a global variable 'speed' as the speed of its wheels.
	This command changes this speed.  The first char is an 's', the next three are the hundreds, tens and ones
	digits of the new speed and the next ten chars are random.

**************************************************************/




/** Includes ************************************************/
#include "HardwareProfile.h"
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#ifndef _MOTORS
	#include "MotorControls.h"
	#define _MOTORS
#endif





/** Defines ***************************************************/
#define phase_a_r LATBbits.LATB12  //Pin 5 NU Board
#define phase_b_r LATBbits.LATB13  //Pin 6 NU Board
#define phase_c_r LATBbits.LATB14  //Pin 7 NU Board
#define phase_d_r LATBbits.LATB15  //Pin 8 NU Board


#define phase_a_l LATAbits.LATA15  //Pin 15 NU Board
#define phase_b_l LATAbits.LATA14  //Pin 16 NU Board
#define phase_c_l LATAbits.LATA3   //Pin 17 NU Board
#define phase_d_l LATAbits.LATA2   //Pin 18 NU Board

#define T2options	T2_ON | T2_PS_1_16 | T2_SOURCE_INT  // These are the setup options for timer two
#define T3options	T3_ON | T3_PS_1_256 | T3_SOURCE_INT  // These are the setup options for timer three
#define T4options	T4_ON | T4_PS_1_256 | T4_SOURCE_INT  // These are the setup options for timer four 

#define BAUDRATE	(115200)	//This is the rate that we will communicate over RS232 at





/** Global Variables **************************************************/
static int left_speed = 0;		// This variable is the current left motor speed in steps/ second
static int right_speed = 0;		// This variable is the current right motor speed in steps/ second
static int	left_steps = 0;		// This variable is used for tracking how many steps the left motor has taken
static int	right_steps = 0;	// This variable is used for tracking how many steps the right motor has taken
static int left_phase = 0;  	// This variable can be any integer between 0 and 3, and will be used for
								// switching the control wires for the left motor in the correct order
static int right_phase = 0;  	// This variable can be any integer between 0 and 3, and will be used for
								// switching the control wires for the right motor in the correct order
static double x_pos = 0.0;		// This variable is what the PIC sees as its current x-position in some world frame
static double y_pos = 0.0;		// This variable is what the PIC sees as its current y-position in some world frame
static double theta = 0.0;		// This variable is what the PIC sees as its current orientation (in radians) w.r.t 
								// the positive global x-axis	
static double L = 2.05;
static double D = 2.15;			
static const double pi = 3.1415926535;		
static double speed = 150;	
static char RS232_In_Buffer[15] = "zzzzzzzzzzzzzzz";	//  This is an array that is initialized with 
														// useless data in it; it is used for temporary 
														// storage of data brought in from the PC on UART2
static int i = 0;			// This is for marking the position in the RS232_In_Buffer that we are writing into.
static int data_flag = 0;  	// This variable is used for telling if we have received updated information about the robot's
							// current position or current instructions.  If we have, we break out of the current control loop
							// and recalculate what we need to do.
static int pose_flag = 0;  	// This flag is used for determining when the SetPose function should be called.  If we are just 
							// sitting in main waiting for an instruction to do, the PoseUpdate function should call SetPose.
							// if we have reached the PoseUpdate function while executing SetPose, SetPose will recursively call
							// itself and PoseUpdate does not need to call SetPose
static int header_flag = 0;			// This flag is used for determining if we have received a header on the receive pins.
static double x_sent = 0.0; 		// This is the value of the x-coordinate received
static double y_sent = 0.0; 		// This is the value of the y-coordinate received
static double ori_sent = 0.0; 		// This is the value of the orientation received
static double first_angle = 0.0; 	// This is the angle of the vector from the robots current position towards its goal position
									// to begin heading towards the new point sent over RS232
static int exec_state = 0;	// This variable is to determine which mode of operation we are currently in:
						// 0) Just sitting there
						// 1) Wheel Speed Control
						// 2) Initial rotation towards destination
						// 3) Driving towards destination
						// 4) Rotating towards final orientation
static int speed_left;		// This is the desired wheel speed of the left motor in steps/sec sent over RS232
static int speed_right;		// This is the desired wheel speed of the right motor in steps/sec sent over RS232						






/** Interrupt Handler Functions:**************************************************************************************************/
// Define this function to be the interrupt service routine for timer 4 (Switches Left Motor)
void __ISR( _TIMER_4_VECTOR, ipl5) T4Interrupt( void) 
{
	// Increment or decrement phase depending on direction
	if (left_speed > 0) 
	{
		left_steps++;
		left_phase--;
		if (left_phase < 0) left_phase = 3;  // Rollover for motor phase
	}
	else 
	{
		left_steps--;
		left_phase++;
		if (left_phase > 3) left_phase = 0;
	}
  
	// Set the current phase on pins C1-C4:
	switch (left_phase)
	{
		case 0:
		{
			phase_a_l = 0;
			phase_b_l = 1;
			phase_c_l = 0;
			phase_d_l = 1;
			break;
		}
		case 1:
		{
			phase_a_l = 0;
			phase_b_l = 1;
			phase_c_l = 1;
			phase_d_l = 0;
			break;
		}
		case 2:
		{
			phase_a_l = 1;
			phase_b_l = 0;
			phase_c_l = 1;
			phase_d_l = 0;
			break;
		}
		case 3:
		{
			phase_a_l = 1;
			phase_b_l = 0;
			phase_c_l = 0;
			phase_d_l = 1;
			break;
		}
	}
	mT4ClearIntFlag();             // clear interrupt flag
}

// Define this function to be the interrupt service routine for timer 3 (Switches Right Motor)
void __ISR( _TIMER_3_VECTOR, ipl5) T3Interrupt( void) 
{
	// Increment or decrement phase depending on direction
	if (right_speed < 0) 
	{
		right_steps--;
		right_phase--;
		if (right_phase < 0) right_phase = 3;  // Rollover for motor phase
	}
	else 
	{
		right_steps++;
		right_phase++;
		if (right_phase > 3) right_phase = 0;
	}

	// Set the current phase on pins D0-D3:
	switch (right_phase)
	{
		case 0:
		{
			phase_a_r = 0;
			phase_b_r = 1;
			phase_c_r = 0;
			phase_d_r = 1;
			break;
		}
		case 1:
		{
			phase_a_r = 0;
			phase_b_r = 1;
			phase_c_r = 1;
			phase_d_r = 0;
			break;
		}
		case 2:
		{
			phase_a_r = 1;
			phase_b_r = 0;
			phase_c_r = 1;
			phase_d_r = 0;
			break;
		}
		case 3:
		{
			phase_a_r = 1;
			phase_b_r = 0;
			phase_c_r = 0;
			phase_d_r = 1;
			break;
		}
	}
	mT3ClearIntFlag();             // clear interrupt flag
}

// UART 2 interrupt handler
// it is set at priority level 2
void __ISR(_UART2_VECTOR, ipl6) IntUart2Handler(void)
{
	char temp;
	// Is this from receiving data?
	if(mU2RXGetIntFlag())
	{
		// Clear the RX interrupt Flag
		mU2RXClearIntFlag();
	
		// Read a char into a temporary variable:
		temp = ReadUART2();
		// Now we need to determine if the data received is one of
		// our header characters.  If it is, we will reset the value 
		// of the buffer counter and flag that we received a header.
		if(temp == 'p'|temp == 'l'|temp == 'r'|temp == 'h'|temp == 's')
		//if(temp == ('p'||'l'||'r'||'h'||'s'))
		{
			i = 0;
			header_flag = 1;
		}
		// Regardless of what temp is, let's place it in the in buffer
		RS232_In_Buffer[i%15] = temp;
		
		// Now let's increment i
		i++;
		
		// If we have received a complete set of data, let's set the flag to
		// zero so that we can deal with the data in main
		if((i-1) == 14 && header_flag == 1) 
		{
			data_flag = 1;
		}
	}

	// Is this interrupt from sending data?
	if ( mU2TXGetIntFlag() )
	{
		mU2TXClearIntFlag();
	}
}

//  In the following function, we will be updating the robot's pose 
// 	using forward kinematics and odometry data.  We will also decide 
//  if we new controls are needed to be sent to the wheels so that 
//  the robot can continue to do what it is supposed to be doing.
void __ISR(_TIMER_2_VECTOR, ipl7) CheckKinematics(void)
{
	double dt = 0.002;
	double Vr = 0.0;
	double Vl = 0.0;
	double omega = 0.0;
	double R = 0.0;
	
	// Now let's get the wheels speeds and convert them into translational velocities
	Vr = (D/2.0)*(2.0*pi/200.0)*((double) GetSpeedRight());
	Vl = (D/2.0)*(2.0*pi/200.0)*((double) GetSpeedLeft());
	
	// Let's calculate the distance to the instantaneous center of rotation and the angular vel.
	R = L*((Vl+Vr)/(Vr-Vl));
	omega = (Vr-Vl)/(2.0*L);
		
	// Now we do the forward kinematics
	if (fabs(R) > 120.0) // This would imply that the robot is essentially going straight
	{
		// First let's calculate how long it has been since this function has been called.
		dt += 0.0000002*(((double) ReadTimer2())+1.0);
		// So, let's perform the straight version kinematics
		x_pos += Vr*dt*cos(theta);
		y_pos += Vr*dt*sin(theta);
	}
	else
	{
		// First let's calculate how long it has been since this function has been called.
		dt += 0.0000002*(((double) ReadTimer2())+1.0);
		// Now let's perform the general kinematics
		x_pos += cos(omega*dt)*R*sin(theta)+sin(omega*dt)*R*cos(theta)-R*sin(theta);
		y_pos += sin(omega*dt)*R*sin(theta)-cos(omega*dt)*R*cos(theta)+R*cos(theta);
		theta += omega*dt;
	}
	
	// Let's force theta to be between 0 and 2pi
	if(theta < 0.0) theta += 2.0*pi;
	if(theta >= 2.0*pi) theta -= 2.0*pi;	
	
	// Are we just controlling wheel speeds?
	if (exec_state == 1)
	{
		// We don't need to check anything then
	}
	// Are we performing an initial rotation to begin heading towards our destination?
	else if (exec_state == 2)
	{
		// Now, let's determine if we have reached the desired orientation yet
		if(fabs(theta-first_angle) <= 0.01 || fabs(fabs(theta-first_angle)-2.0*pi) <= 0.01)
		{
			theta = first_angle;
			exec_state = 3;
			pose_flag = 1;
		}
	}
	else if (exec_state == 3)
	{
		// Now, are we there yet?
		if(fabs(x_sent-x_pos) < 0.05 && fabs(y_sent-y_pos) < 0.05)
		{
			x_pos = x_sent;
			y_pos = y_sent;
			SetSpeedLeft(0);
			SetSpeedRight(0);
			exec_state = 4;
			pose_flag = 1;
		}
	}
	else if (exec_state == 4)
	{	
		// Now, let's determine if we have reached the final orientation yet
		if(fabs(theta-ori_sent) <= 0.01 || fabs(fabs(theta-ori_sent)-2.0*pi) <= 0.01)
		{
			theta = ori_sent;
			// Since we have, we don't need to keep updating the kinematics
			exec_state = 0;
			pose_flag = 0;
			// Also, let's stop the motors:
			SetSpeedLeft(0);
			SetSpeedRight(0);
			// And disable the H-Bridges:
			StopHBridges();
		}
	}
	mT2ClearIntFlag(); 		// clear interrupt flag
}



/** User Called Functions ********************************************************************************/
// Following function returns the number of 
// steps that the left motor has taken:
int GetStepsLeft(void)
{
  return left_steps;
}

// Following function returns the number of 
// steps that the right motor has taken:
int GetStepsRight(void)
{
  return right_steps;
}


// The following function returns the current speed 
// of the left motor
int GetSpeedLeft(void)
{
  return left_speed;
}

// The following function returns the current speed 
// of the left motor
int GetSpeedRight(void)
{
  return right_speed;
}

// Next function allows to user to re-set a "home" position by
// manually changing the number of steps for the left motor:
void SetStepsLeft(int set_steps)
{
	INTEnable(INT_T4, 0);
	left_steps = set_steps;
	INTEnable(INT_T4, 1);
}

// Next function allows to user to re-set a "home" position by
// manually changing the number of steps for the right motor:
void SetStepsRight(int set_steps)
{
	INTEnable(INT_T3, 0);
	right_steps = set_steps;
	INTEnable(INT_T3, 1);
}


// This function allows the user to specify the desired left motor speed:
void SetSpeedLeft(int motor_speed)  // motor speed in steps/s
{
	left_speed = motor_speed;
	if (motor_speed == 0)
	{
		// Turn off timer if motor is stopped:
		CloseTimer4();
	}
	else
	{
		int Per;
		Per = (80000000/abs(left_speed))/256-1;  // set the value in the period register
		OpenTimer4(T4options, Per);
		mT4SetIntPriority( 5); 	// set Timer 4 Interrupt Priority
		mT4ClearIntFlag(); 		// clear interrupt flag
		mT4IntEnable( 1);		// enable timer 4 interrupts
	}
}


// This function allows the user to specify the desired right motor speed:
void SetSpeedRight(int motor_speed)  // motor speed in steps/s
{
	right_speed = motor_speed;
	if (motor_speed == 0)
	{
		// Turn off timer if motor is stopped:
		CloseTimer3();
	}
	else
	{
		int Per;
		Per = (80000000/abs(right_speed))/256-1;  // set the value in the period register
		OpenTimer3(T3options, Per);
		mT3SetIntPriority( 5); 	// set Timer3 Interrupt Priority
		mT3ClearIntFlag(); 		// clear interrupt flag
		mT3IntEnable( 1);		// enable timer3 interrupts
	}
}

// This function is for disabling the H-bridges:
void StopHBridges(void)
{
	// Turn off timer 3 and 4:
	CloseTimer3();
	CloseTimer4();
	// Also, let's turn off all of the phases to both motors:
	phase_a_r = 0;
	phase_b_r = 0;
	phase_c_r = 0;
	phase_d_r = 0;

	phase_a_l = 0;
	phase_b_l = 0;
	phase_c_l = 0;
	phase_d_l = 0;
}

// This function is used for initilizing the UART2
void initUART2(int pbClk)
{
	// define setup Configuration 1 for OpenUARTx
		// Module Enable 
		// Work in IDLE mode 
		// Communication through usual pins 
		// Disable wake-up 
		// Loop back disabled 
		// Input to Capture module from ICx pin 
		// no parity 8 bit 
		// 1 stop bit 
		// IRDA encoder and decoder disabled 
		// CTS and RTS pins are disabled 
		// UxRX idle state is '1' 
		// 16x baud clock - normal speed
	#define config1 	UART_EN | UART_IDLE_CON | UART_RX_TX | UART_DIS_WAKE | UART_DIS_LOOPBACK | UART_DIS_ABAUD | UART_NO_PAR_8BIT | UART_1STOPBIT | UART_IRDA_DIS | UART_DIS_BCLK_CTS_RTS| UART_NORMAL_RX | UART_BRGH_SIXTEEN
	
	// define setup Configuration 2 for OpenUARTx
		// IrDA encoded UxTX idle state is '0'
		// Enable UxRX pin
		// Enable UxTX pin
		// Interrupt on transfer of every character to TSR 
		// Interrupt on every char received
		// Disable 9-bit address detect
		// Rx Buffer Over run status bit clear
	#define config2		UART_TX_PIN_LOW | UART_RX_ENABLE | UART_TX_ENABLE | UART_INT_TX | UART_INT_RX_CHAR | UART_ADR_DETECT_DIS | UART_RX_OVERRUN_CLEAR	

	// Open UART2 with config1 and config2
	OpenUART2( config1, config2, pbClk/16/BAUDRATE-1);	// calculate actual BAUD generate value.
		
	// Configure UART2 RX Interrupt with priority 7
	ConfigIntUART2(UART_INT_PR6 | UART_RX_INT_EN);
}								


void InitTimer2(void)
{
	int Per, frequency;
	frequency  = 500; // Let's check the kinematics this many times per second
	Per = (80000000/frequency)/16-1;  // Set the value in the period register
	OpenTimer2(T2options, Per);	
	mT2SetIntPriority( 7); 	// set Timer 2 Interrupt Priority
	mT2ClearIntFlag(); 		// clear interrupt flag
	mT2IntEnable( 1);		// enable timer 2 interrupts
}


void SetPose(double xdest,double ydest,double thdest)
{
	INTEnable(INT_T2,0);
	
	
	// Are we performing the initial orientation?
	if(exec_state == 2)
	{
		// First, let's calculate the angle that we need to rotate through to 
		// begin heading towards the desired point:
		if(fabs(xdest-x_pos) < 0.000001 && fabs(ydest-y_pos) < 0.000001)
		{
			first_angle = theta;
		}	
		else
		{
			first_angle = (atan2((ydest-y_pos),(xdest-x_pos)));	
		}

		if(first_angle <= 0.0) first_angle += 2.0*pi;
		
		// Let's let main know that it can stop calling this function now
		pose_flag = 0;
		// Now let's set the motor speeds such that we perform the first rotation correctly:
		if((first_angle-theta >= 0.0)&&(first_angle-theta <= pi))
		{
			SetSpeedRight((int) speed);
			SetSpeedLeft((int) -speed);
		}
		else if((first_angle-theta >= 0.0)&&(first_angle-theta > pi))
		{
			SetSpeedRight((int) -speed);
			SetSpeedLeft((int) speed);
		}
		else if((first_angle-theta < 0.0)&&(first_angle-theta >= -pi))
		{	
			SetSpeedRight((int) -speed);
			SetSpeedLeft((int) speed);
		}
		else if((first_angle-theta < 0.0)&&(first_angle-theta < -pi))
		{	
			SetSpeedRight((int) speed);
			SetSpeedLeft((int) -speed);
		}
	}
	// Are we driving to the destination?
	else if(exec_state == 3)
	{
		// Let's let main know that it can stop calling this function now
		pose_flag = 0;	
		// Now let's set the motor speeds for traversing the distance:
		SetSpeedRight((int) speed);
		SetSpeedLeft((int) speed);
	}
	else if(exec_state == 4)
	{
		// Let's let main know that it can stop calling this function now
		pose_flag = 0;	
		// Are we going to the final orientation:
		if((thdest-theta >= 0.0)&&(thdest-theta <= pi))
		{
			SetSpeedRight((int) speed);
			SetSpeedLeft((int) -speed);
		}
		else if((thdest-theta >= 0.0)&&(thdest-theta > pi))
		{
			SetSpeedRight((int) -speed);
			SetSpeedLeft((int) speed);
		}
		else if((thdest-theta < 0.0)&&(thdest-theta >= -pi))
		{		
			SetSpeedRight((int) -speed);
			SetSpeedLeft((int) speed);
		}
		else if((thdest-theta < 0.0)&&(thdest-theta < -pi))
		{
			SetSpeedRight((int) speed);
			SetSpeedLeft((int) -speed);
		}
	}
	
	INTEnable(INT_T2,1);
}



// This is the function that is called after a complete instruction has been received.  It decodes that instruction
// and then sets important global variables to the correct values.
void PoseUpdate(void)
{
	// The first thing that we do is clear the flags for receiving header bits and for receiving new instructions:
	data_flag = 0;
	header_flag = 0;

	// Now, let's disable both timer interrupts and the UART interrupt so that they are not interrupting while 
	//we are sorting out the data that was received:
	INTEnable(INT_T4, 0);
	INTEnable(INT_T3, 0);	
	INTEnable(INT_T2, 0);
	INTEnable(INT_U2RX, 0);
	
	// Now, we can begin sorting the data:
	char x_sign; 	// Determines if the received x-coordinate is positive or negative
	char y_sign; 	// Determines if the received y-coordinate is positive or negative
	char data;   	// This is the very first entry in the in buffer (the header byte)
	char dir_left;	// The direction of the left wheel that was sent
	char dir_right; // The direction of the right wheel that was sent
		
	// Set the current byte of the RS232_In_Buffer to be equal to data:
	data = RS232_In_Buffer[0];

	// If the header is a 'p', we have received an instruction for where to go:
	if (data == 'p')
	{
		exec_state = 2;
		// Now let's convert the 15 bytes of chars into useful data
		x_sign = RS232_In_Buffer[1];
		x_sent = ((double)(1000*((int) RS232_In_Buffer[2]-48)+
			100*((int) RS232_In_Buffer[3]-48)+10*((int) RS232_In_Buffer[4]-48)+
				((int) RS232_In_Buffer[5]-48)))/100.0;
		y_sign = RS232_In_Buffer[6];
		y_sent = ((double)(1000*((int) RS232_In_Buffer[7]-48)+
			100*((int) RS232_In_Buffer[8]-48)+10*((int) RS232_In_Buffer[9]-48)+
				((int) RS232_In_Buffer[10]-48)))/100.0;
		ori_sent = ((double)(1000*((int) RS232_In_Buffer[11]-48)+
			100*((int) RS232_In_Buffer[12]-48)+10*((int) RS232_In_Buffer[13]-48)+
				((int) RS232_In_Buffer[14]-48)))/1000.0;

		if(x_sign == '0') x_sent = -x_sent;
		if(y_sign == '0') y_sent = -y_sent;
		
		// Let's set pose_flag so that we know we need to call SetPose()
		pose_flag = 1;
	}
	
	// If the header byte is an 'l', we have received updated information about the 
	// robot's current location.
	else if(data == 'l')
	{
		x_sign = RS232_In_Buffer[1];
		x_pos = ((double)(1000*((int) RS232_In_Buffer[2]-48)+
			100*((int) RS232_In_Buffer[3]-48)+10*((int) RS232_In_Buffer[4]-48)+
				((int) RS232_In_Buffer[5]-48)))/100.0;
		y_sign = RS232_In_Buffer[6];
		y_pos = ((double)(1000*((int) RS232_In_Buffer[7]-48)+
			100*((int) RS232_In_Buffer[8]-48)+10*((int) RS232_In_Buffer[9]-48)+
				((int) RS232_In_Buffer[10]-48)))/100.0;
		theta = ((double)(1000*((int) RS232_In_Buffer[11]-48)+
			100*((int) RS232_In_Buffer[12]-48)+10*((int) RS232_In_Buffer[13]-48)+
				((int) RS232_In_Buffer[14]-48)))/1000.0;

		if(x_sign == '0') x_pos = -x_pos;
		if(y_sign == '0') y_pos = -y_pos;
		
		// So, we have just received an updated position and orientation, we need to 
		// decide if it is worth it to fix the error.
		
		// If we are currently executing a point command, and we are currently translating,
		// it is very easy to make an adjustment:
		if (exec_state == 3) 
		{
			pose_flag = 1;
			exec_state = 2;
		}
		
		// If we are done executing a point command and basically just sitting waiting for 
		// the next, and our error is bad enough, we can go ahead and try to fix it
		if (exec_state == 0)
		{
			if((pow((x_pos-x_sent),2.0)+pow((y_pos-y_sent),2.0) > 3.0) || (fabs(theta-ori_sent) > 1.571))
			{
				pose_flag = 1;
				exec_state = 2;
			}
		}
	}
	else if (data == 'r')
	{
		SetSpeedLeft(0);
		SetSpeedRight(0);
		StopHBridges();
		x_pos = 0.0;
		y_pos = 0.0;
		theta = 0.0;
		while(BusyUART2());
		putsUART2("Coordinate System Reset\r\n");
	}
	// If we receive an h as the header char, then that means that we will simply control the wheel speeds
	else if (data == 'h')
	{
		exec_state = 1;
		dir_left = RS232_In_Buffer[1];
		speed_left = 100*((int) RS232_In_Buffer[2]-48)+10*((int) RS232_In_Buffer[3]-48)+((int) RS232_In_Buffer[4]-48);
		dir_right = RS232_In_Buffer[5];
		speed_right = 100*((int) RS232_In_Buffer[6]-48)+10*((int) RS232_In_Buffer[7]-48)+((int) RS232_In_Buffer[8]-48);

		// Here we need to calculate the motor speeds and set them	
		if(dir_left == '1')
			SetSpeedLeft(speed_left);
		else
			SetSpeedLeft(-speed_left);
		
		if(dir_right == '1')
			SetSpeedRight(speed_right);
		else
			SetSpeedRight(-speed_right);	

		// We just received commands for explicitly controlling the wheel speeds, let's force the pose control
		// to stop executing:
		pose_flag = 0;
	}
	else if (data == 's')
	{
		speed = 100*((int) RS232_In_Buffer[1]-48)+10*((int) RS232_In_Buffer[2]-48)+((int) RS232_In_Buffer[3]-48);
		while(BusyUART2());
		putsUART2("Changed Default Speed\r\n");
	}
	// Now, let's re-enable all interrupts:
	INTEnable(INT_T4, 1);
	INTEnable(INT_T3, 1);	
	INTEnable(INT_T2,1);
	INTEnable(INT_U2RX, 1);
}

void RuntimeOperation(void)
{
	// If the buffer has just filled, and we know we received a good
	// set of data (because it began with a header bit and was 14 chars), 
	// let's call our functions that interpret and execute the received 
	// data
	if(data_flag == 1) 
	{		
		PoseUpdate();
	}
	
	// If we are currently controlling pose, lets call that function
	if(pose_flag == 1) SetPose(x_sent,y_sent,ori_sent);
}
